<?php

use WooDostavista\Enums\ScheduleTimeEnum;
use WooDostavista\ModuleSettings\ModuleSettings;
use WooDostavista\View\JsonResponse;
use WooDostavista\View\View;
use WooDostavista\Warehouses\Warehouse;
use WooDostavista\Warehouses\WarehouseManager;
use WooDostavista\ModuleConfig\ModuleConfig;
use WooDostavista\GeoCoder\GeoCoderService;
use WooDostavista\GeneralCache\Cache;
use WooDostavista\GeoCoder\Providers\GeoProvidersEnum;
use WooDostavista\DvCmsModuleApiClient\DvCmsModuleApiClient;
use WooDostavista\GeoCoder\Providers\DostavistaGeoCoderClient;
use WooDostavista\GeoCoder\Providers\DadataGeoCoderClient;
use WooDostavista\Http\HttpClient;
use WooDostavista\GeoCoder\GeoCoderHttpException;

defined('ABSPATH') || exit;

class WC_Dostavista_Warehouses_Controller
{
    public static function index()
    {
        global $wpdb;

        // Пропустим запрос через милвару, проверяющую  законченность визарда
        WC_Dostavista_Middlewares::preventUnfinishedWizard();

        $warehouseManager = new WarehouseManager($wpdb);
        $warehouses       = $warehouseManager->getList();

        $settings = new ModuleSettings($wpdb);

        $view = new View(
            __DIR__ . '/../views/warehouse-list.php',
            [
                'warehouses'         => $warehouses,
                'defaultWarehouseId' => $settings->getDefaultPickupWarehouseId(),
            ]
        );

        echo $view->getRenderedHtml();
    }

    public static function edit()
    {
        global $wpdb;

        // Пропустим запрос через милвару, проверяющую  законченность визарда
        WC_Dostavista_Middlewares::preventUnfinishedWizard();

        $warehouseManager = new WarehouseManager($wpdb);
        $moduleConfig = new ModuleConfig();

        $id = ($_REQUEST['id'] ?? null);
        if ($id) {
            $warehouse = $warehouseManager->getById((int) $id);
        } else {
            $warehouse = new Warehouse();
        }

        $scheduleTimeEnum = ScheduleTimeEnum::getEnum();

        $view = new View(
            __DIR__ . '/../views/warehouse-edit.php',
            [
                'warehouse'                => $warehouse,
                'scheduleTimeEnum'         => $scheduleTimeEnum,
                'isAddressSuggestionsUsed' => $moduleConfig->isUseAddressSuggestions(),
            ]
        );

        echo $view->getRenderedHtml();
    }

    public static function store()
    {
        global $wpdb;

        $warehouseManager = new WarehouseManager($wpdb);

        $postBody = file_get_contents('php://input');
        $data     = json_decode($postBody, true) ?? [];

        if (!$data) {
            (new JsonResponse(['error' => 'required_post_data'], 500))->render();
            exit();
        }

        $id = ($data['id'] ?? null);
        if ($id) {
            $warehouse = $warehouseManager->getById((int) $id);
        } else {
            $warehouse = new Warehouse();
        }

        $oldAddress = $warehouse->address;

        $warehouseCity         = trim($data['city']);
        $warehouseDefaultTitle = WC_Dostavista_Lang::getHtml('warehouse_default_title');

        $warehouse->name           = trim($data['name'] ?: "{$warehouseDefaultTitle} {$warehouseCity}");
        $warehouse->city           = $warehouseCity;
        $warehouse->address        = trim($data['address']);
        $warehouse->workStartTime  = trim($data['work_start_time']);
        $warehouse->workFinishTime = trim($data['work_finish_time']);
        $warehouse->contactName    = trim($data['contact_name']);
        $warehouse->contactPhone   = trim($data['contact_phone']);
        $warehouse->note           = trim($data['note']);

        $warehouseManager->save($warehouse);

        // Если адрес был изменен, то получим координаты нового адреса
        if ($oldAddress !== $warehouse->address || !((int) $warehouse->geoLat)) {
            self::updateWarehouseGeoCoordinates($warehouse);
        }

        (new JsonResponse(
            [
                'redirect_url' => admin_url('admin.php?page=woo_dostavista_warehouses'),
                'warehouse' => [
                    'id' => $warehouse->id,
                ],
            ]
        ))->render();
    }

    public static function delete()
    {
        global $wpdb;

        $warehouseManager = new WarehouseManager($wpdb);

        $postBody = file_get_contents('php://input');
        $data     = json_decode($postBody, true) ?? [];

        $id = ($data['id'] ?? null);
        $warehouseManager->delete((int) $id);

        (new JsonResponse([]))->render();
    }

    private static function updateWarehouseGeoCoordinates(Warehouse $warehouse)
    {
        global $wpdb;
        $warehouseManager = new WarehouseManager($wpdb);

        $geoCoderProvider = null;

        $dvServiceLocator = WC_Dostavista_Service_Locator::getInstance();
        $geoCoderService = $dvServiceLocator->getGeoCoderService();

        try {
            $geoPoint = $geoCoderService->getGeoPointByAddress($warehouse->getFullAddress());

            if (!$geoPoint) {
                return;
            }

            $warehouse->geoLat = $geoPoint->getLatitude();
            $warehouse->geoLon = $geoPoint->getLongitude();
            $warehouse->city   = $geoPoint->getCityName() ?? $warehouse->city;

            $warehouseManager->save($warehouse);
        } catch (GeoCoderHttpException $e) {
            return;
        }
    }
}
