<?php

namespace WooDostavista\Warehouses;

use stdClass;
use WooDostavista\DvCmsModuleApiClient\DvCmsModuleApiClient;
use WooDostavista\GeneralCache\Cache;
use WooDostavista\GeoCoder\GeoCoderService;
use WooDostavista\GeoCoder\Providers\DadataGeoCoderClient;
use WooDostavista\GeoCoder\Providers\DostavistaGeoCoderClient;
use WooDostavista\GeoCoder\Providers\GeoProvidersEnum;
use WooDostavista\Http\HttpClient;
use WooDostavista\ModuleConfig\ModuleConfig;
use WooDostavista\ModuleSettings\ModuleSettings;
use WooDostavista\Utils\GeoDistance;
use wpdb;

class WarehouseManager
{
    /** @var wpdb */
    private $wpdb;

    public function __construct(wpdb $wpdb)
    {
        $this->wpdb = $wpdb;
        $this->createTablesIfNotExists();
    }

    public function createTablesIfNotExists()
    {
        $this->wpdb->query(
            "
                CREATE TABLE IF NOT EXISTS `woo_dostavista_warehouses` (
                    `id` INT(11) NOT NULL AUTO_INCREMENT,
                    `name` VARCHAR(255) DEFAULT '',
                    `address` VARCHAR(255) DEFAULT '',
                    `work_start_time` VARCHAR(5) DEFAULT '08:00',
                    `work_finish_time` VARCHAR(5) DEFAULT '20:00',
                    `contact_name` VARCHAR(255) DEFAULT '',
                    `contact_phone` VARCHAR(100) DEFAULT '',
                    `note` TEXT DEFAULT '',
                    PRIMARY KEY (`id`)
                ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci;
            "
        );

        // Alter table with update
        $rows = $this->wpdb->get_results("SHOW COLUMNS FROM `woo_dostavista_warehouses` LIKE 'city'");
        if (!$rows) {
            $this->wpdb->query("ALTER TABLE `woo_dostavista_warehouses` ADD COLUMN `city` VARCHAR(255) DEFAULT ''");
        }

        // С версии 1.3 склады хранят координаты
        $geoLatRows = $this->wpdb->get_results("SHOW COLUMNS FROM `woo_dostavista_warehouses` LIKE 'geo_lat'");
        $geoLonRows = $this->wpdb->get_results("SHOW COLUMNS FROM `woo_dostavista_warehouses` LIKE 'geo_lon'");
        $isWarehouseCordinatesUpdateRequired = false;
        if (!$geoLatRows) {
            $isWarehouseCordinatesUpdateRequired = true;
            $this->wpdb->query("ALTER TABLE `woo_dostavista_warehouses` ADD COLUMN `geo_lat` DECIMAL(9,6) DEFAULT NULL");
        }
        if (!$geoLonRows) {
            $isWarehouseCordinatesUpdateRequired = true;
            $this->wpdb->query("ALTER TABLE `woo_dostavista_warehouses` ADD COLUMN `geo_lon` DECIMAL(9,6) DEFAULT NULL");
        }

        if ($isWarehouseCordinatesUpdateRequired) {
            $warehouses = $this->getList();

            $cache      = new Cache($this->wpdb);
            $config     = new ModuleConfig();
            $settings   = new ModuleSettings($this->wpdb);
            $httpClient = new HttpClient();

            $geoCoderProvider = null;

            switch ($config->getGeoCoderProvider()) {
                case GeoProvidersEnum::PROVIDER_DOSTAVISTA:
                    $dvCmsModuleApiClient = new DvCmsModuleApiClient(
                        $settings->getApiUrl(), $settings->getAuthToken()
                    );

                    $geoCoderProvider = new DostavistaGeoCoderClient($dvCmsModuleApiClient);
                    break;
                case GeoProvidersEnum::PROVIDER_DADATA:
                    $geoCoderProvider = $config->getDadataToken()
                        ? new DadataGeoCoderClient($httpClient, $config->getDadataToken())
                        : null;
                    break;
            }

            if ($geoCoderProvider) {
                $geoCoderService = new GeoCoderService($geoCoderProvider, $cache);

                foreach ($warehouses as $warehouse) {
                    $this->updateWarehouseGeoCoordinates($warehouse, $geoCoderService);
                }
            }
        }
    }

    public function save(Warehouse $warehouse)
    {
        if ($warehouse->id) {
            $this->wpdb->query(
                $this->wpdb->prepare(
                    "
                        UPDATE `woo_dostavista_warehouses`
                        SET 
                            `name` = %s,
                            `city` = %s,
                            `address` = %s,
                            `work_start_time` = %s,
                            `work_finish_time` = %s,
                            `contact_name` = %s,
                            `contact_phone` = %s,
                            `note` = %s,
                            `geo_lat` = %s,
                            `geo_lon` = %s
                        WHERE `id` = %d
                    ",
                    $warehouse->name,
                    $warehouse->city,
                    $warehouse->address,
                    $warehouse->workStartTime,
                    $warehouse->workFinishTime,
                    $warehouse->contactName,
                    $warehouse->contactPhone,
                    $warehouse->note,
                    $warehouse->geoLat,
                    $warehouse->geoLon,
                    $warehouse->id
                )
            );
        } else {
            $this->wpdb->query(
                $this->wpdb->prepare(
                    "
                        INSERT INTO `woo_dostavista_warehouses` 
                          (`name`, `city`, `address`, `work_start_time`, `work_finish_time`, `contact_name`, `contact_phone`, `note`, `geo_lat`, `geo_lon`)
                        VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s) 
                    ",
                    $warehouse->name,
                    $warehouse->city,
                    $warehouse->address,
                    $warehouse->workStartTime,
                    $warehouse->workFinishTime,
                    $warehouse->contactName,
                    $warehouse->contactPhone,
                    $warehouse->note,
                    $warehouse->geoLat,
                    $warehouse->geoLon
                )
            );

            $warehouse->id = (int) $this->wpdb->insert_id;
        }
    }

    /**
     * @param int $id
     * @return Warehouse|null
     */
    public function getById(int $id)
    {
        $warehouses = $this->getByIds([$id]);
        if ($warehouses) {
            return $warehouses[0];
        }

        return null;
    }

    /**
     * @param array $ids
     * @return Warehouse[]
     */
    public function getByIds(array $ids): array
    {
        if (!$ids) {
            return [];
        }

        $warehouses = [];

        $idsWhereSql = join(',', $ids);
        $rows = $this->wpdb->get_results("SELECT * FROM `woo_dostavista_warehouses` WHERE `id` IN ({$idsWhereSql}) ORDER BY `id` ASC");
        if ($rows) {
            foreach ($rows as $row) {
                $warehouses[] = $this->populateModel($row);
            }
        }

        return $warehouses;
    }

    /**
     * @return Warehouse[]
     */
    public function getList(): array
    {
        $warehouses = [];

        $rows = $this->wpdb->get_results("SELECT * FROM `woo_dostavista_warehouses` ORDER BY `id` ASC");
        if ($rows) {
            foreach ($rows as $row) {
                $warehouses[] = $this->populateModel($row);
            }
        }

        return $warehouses;
    }

    public function delete(int $id): bool
    {
        $warehouse = $this->getById($id);
        if (!$warehouse) {
            return false;
        }

        $this->wpdb->query("DELETE FROM `woo_dostavista_warehouses` WHERE `id` = " . (int) $warehouse->id);

        return true;
    }

    private function populateModel(stdClass $row): Warehouse
    {
        $warehouse = new Warehouse();
        $warehouse->id             = (int) $row->id;
        $warehouse->name           = $row->name;
        $warehouse->city           = $row->city;
        $warehouse->address        = $row->address;
        $warehouse->workStartTime  = date('H:i', strtotime($row->work_start_time));
        $warehouse->workFinishTime = date('H:i', strtotime($row->work_finish_time));
        $warehouse->contactName    = $row->contact_name;
        $warehouse->contactPhone   = $row->contact_phone;
        $warehouse->note           = $row->note;
        $warehouse->geoLat           = $row->geo_lat;
        $warehouse->geoLon           = $row->geo_lon;

        return $warehouse;
    }

    /**
     * @return Warehouse|null
     */
    public function getFirstItemByCityName(string $cityName)
    {
        $data = $this->wpdb->get_results("SELECT * FROM `woo_dostavista_warehouses` WHERE city = '{$cityName}' LIMIT 1");
        if ($data && isset($data[0])) {
            return $this->populateModel($data[0]);
        }
        return null;
    }

    public function updateWarehouseGeoCoordinates(Warehouse $warehouse, GeoCoderService $geoCoderService): bool
    {
        $address = $warehouse->getFullAddress();

        $geoPoint = $geoCoderService->getGeoPointByAddress($address);
        if (!$geoPoint) {
            return false;
        }

        $warehouse->geoLat = $geoPoint->getLatitude();
        $warehouse->geoLon = $geoPoint->getLongitude();

        $this->save($warehouse);

        return true;
    }

    /**
     * @return Warehouse|null
     */
    public function getNearestWarehouseByAddress(string $address, GeoCoderService $geoCoderService)
    {
        $warehouses = $this->getList();

        // Получим координаты адреса
        $addressGeoPoint = $geoCoderService->getGeoPointByAddress($address);
        if (!$addressGeoPoint) {
            return null;
        }

        $minDistanceMeters = 9999999999;
        $nearestWarehouse  = null;

        foreach ($warehouses as $warehouse) {
            if ($warehouse->geoLon && $warehouse->geoLat) {
                $warehouseGeoLat = $warehouse->geoLat;
                $warehouseGeoLon = $warehouse->geoLon;
            } else {
                $warehouseGeoPoint = $geoCoderService->getGeoPointByAddress($warehouse->getFullAddress());
                if (!$warehouseGeoPoint || !($warehouseGeoPoint->getLatitude() && $warehouseGeoPoint->getLongitude())) {
                    continue;
                }

                $this->updateWarehouseGeoCoordinates($warehouse, $geoCoderService);

                $warehouseGeoLat = $warehouseGeoPoint->getLatitude();
                $warehouseGeoLon = $warehouseGeoPoint->getLongitude();
            }

            $distanceInMeters = GeoDistance::getDistanceInMeters(
                $addressGeoPoint->getLatitude(),
                $addressGeoPoint->getLongitude(),
                $warehouseGeoLat,
                $warehouseGeoLon
            );

            if ($distanceInMeters < $minDistanceMeters) {
                $minDistanceMeters = $distanceInMeters;
                $nearestWarehouse  = $warehouse;
            }
        }

        return $nearestWarehouse;
    }
}
